@@ -3,7 +3,7 @@
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Donald Ephraim Curtis
 -- @copyright 2008 Julien Danjou
--- @release v3.5.5
+-- @release v3.5.8
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -74,7 +74,7 @@ local function tile_group(cls, wa, orientation, fact, group)
 end
 
 local function do_tile(param, orientation)
-    local t = tag.selected(param.screen)
+    local t = param.tag or tag.selected(param.screen)
     orientation = orientation or "right"
 
     -- This handles all different orientations.
